<?php

/* 
 * Copyright (C) ---Put here your own copyright and developer email---
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 *  \file       
 *  \ingroup    
 *  \brief      This file is an example for a CRUD class file (Create/Read/Update/Delete)
 * 			
 */


/**
 * 	Put here description of your class
 */
class DolCardDav
{

    public $db;       //!< To store db handler
    /**
			@var 
    */
    public $error;       //!< To return error code (or message)
    /**
			@var 
    */
    public $errors = array();    //!< To return several error codes (or messages)
    /**
			@var 
    */
    public $id;
     /**
			@var varchar id in webdav server 
    */
    public $carddav_id;
    /**
			@var varchar unique id for this card; this value is identifical in mutliple server
    */
    public $carddav_etag;
    /**
			@var 
    */
    public $fk_socid;
    /**
			@var 
    */
    public $fk_contactid;
    /**
			@var 
    */
    public $last_modified;
    /**
			@var 
    */
    public $statut;
    /**
			@var 
    */
    public $fk_user;
    /**
			@var 
    */
    public $updateprogress; 

    /**
     *  Constructor
     *
     *  @param	DoliDb		$db      Database handler
     */
    function __construct($db)
    {
        $this->db = $db;
        return 1;
    }

    /**
     *  Create object into database
     *
     *  @param	User	$user        User that creates
     *  @param  int		$notrigger   0=launch triggers after, 1=disable triggers
     *  @return int      		   	 <0 if KO, Id of created object if OK
     */
    function create($user, $notrigger = 0)
    {
        global $conf, $langs;
        $error = 0;

        if (isset($this->carddav_etag))
						$this->carddav_etag = trim($this->carddav_etag);
				if (isset($this->carddav_id))
						$this->carddav_id = trim($this->carddav_id);
        if (isset($this->fk_socid))
						$this->fk_socid = (int)$this->fk_socid;
        if (isset($this->fk_contactid))
						$this->fk_contactid = (int)$this->fk_contactid;
        if (isset($this->last_modified))
						$this->last_modified = trim($this->last_modified);
        if (isset($this->statut))
						$this->statut = (int)$this->statut;
        if (isset($this->fk_user)) 
						$this->fk_user = trim($this->fk_user);
				if (isset($this->fk_user_carddav)) 
						$this->fk_user_carddav = trim($this->fk_user_carddav);


        $sql = "INSERT INTO " . MAIN_DB_PREFIX . "carddav_link(";
				$sql.= "carddav_etag,";
				$sql.= "carddav_id,";
				$sql.= "fk_socid,";
        $sql.= "fk_contactid,";
        $sql.= "last_modified,";
        $sql.= "statut,";
        $sql.= "fk_user_carddav,";
        $sql.= "fk_user,";
				$sql.= "updateprogress";
        $sql.= ") VALUES (";
        $sql.= " " . (!isset($this->carddav_etag) ? 'NULL' : "'" . $this->db->escape($this->carddav_etag) . "'") . ",";
        $sql.= " " . (!isset($this->carddav_id) ? 'NULL' : "'" . $this->db->escape($this->carddav_id) . "'") . ",";
        $sql.= " " . (!isset($this->fk_socid) ? 'NULL' : "'" . (int)$this->fk_socid . "'") . ",";
        $sql.= " " . (!isset($this->fk_contactid) ? 'NULL' : "'" . (int)$this->fk_contactid . "'") . ",";
        $sql.= " " .  'NOW()' . ",";
        $sql.= " " . (!isset($this->statut) ? 'NULL' : "'" . (int)$this->statut . "'") . ",";
        $sql.= " " . (!isset($this->fk_user_carddav) ? 'NULL' : "'" . $this->fk_user_carddav . "'") . ",";
        $sql.= " " . (!isset($this->fk_user) ? 'NULL' : "'" . $this->fk_user . "'") . ",";
        $sql.= " 1 ";
        $sql.= ")";
// echo $sql; 
        $this->db->begin();

        dol_syslog(get_class($this) . "::create sql=" . $sql, LOG_DEBUG);
        $resql = $this->db->query($sql);
        if (!$resql)
        {
            $error++;
            $this->errors[] = "Error " . $this->db->lasterror();
        }

        if (!$error)
        {
            $this->id = $this->db->last_insert_id(MAIN_DB_PREFIX . "carddav_link");

            if (!$notrigger)
            {
            }
        }


        if ($error)
        {
            foreach ($this->errors as $errmsg)
            {
                dol_syslog(get_class($this) . "::create " . $errmsg, LOG_ERR);
                $this->error.=($this->error ? ', ' . $errmsg : $errmsg);
            }
            $this->db->rollback();
            return -1 * $error;
        }
        else
        {
            $this->db->commit();
            return $this->id;
        }
    }

    /**
     *  Load object in memory from the database
     *
     *  @param int		$fk_user    id of user to load
     *  @param string $etag indice 
     *  @param int		$fk_socid
     *  @param int $fk_contactid
     *  @param $extid carddav id is unique id generated by serveer carddav 
     *  @param bool $inprocess true for specific no current rows
     *  @param int $fk_user_carddav is rowid user_carddav  table
     *  @param int $number, is count number of link existing carddav account 
     *  @return int          	<0 if KO, >0 if OK
     */
    function fetch_from($fk_user, $etag='', $fk_socid=0, $fk_contactid=0 , $extid='',$inprocess=false, $fk_user_carddav=0/*, $number=0*/)
    {
        global $langs;
        $sql = "SELECT";
        $sql.= " t.rowid,";
				$sql.= " t.carddav_etag,";
				$sql.= " t.carddav_id,";
				$sql.= " t.fk_socid,";
        $sql.= " t.fk_contactid,";
        $sql.= " t.last_modified,";
        $sql.= " t.statut,";
				$sql.= " t.fk_user_carddav,";
        $sql.= " t.fk_user,";
        $sql.= " t.updateprogress";

        $sql.= " FROM " . MAIN_DB_PREFIX . "carddav_link as t";
        $sql.= " WHERE t.fk_user = '" . (int)$fk_user."' "; 
        
				if($fk_user_carddav > 0) 
					$sql.= " AND t.fk_user_carddav = '" . (int)$fk_user_carddav."' ";
// 				elseif($number > 0) 
// 					$sql.= " AND t.number = '" . (int)$number."' ";
        
        if(!empty($etag)) 
					$sql.= " AND t.carddav_etag = '" . $etag."'  ";
				if(!empty($extid)) 
					$sql.= " AND t.carddav_id = '" . $extid."'  ";
				if($fk_socid > 0) 
					$sql.= " AND t.fk_socid = '" . $fk_socid."'  ";
				if($fk_contactid > 0) 
					$sql.= " AND t.fk_contactid = '" . $fk_contactid."'  ";
					
				if($inprocess !=false) 
					$sql.= " AND t.updateprogress = 0 ";
					
				$sql .=" ORDER BY statut DESC";
// echo $sql; 
// exit;
        $resql = $this->db->query($sql);
        if ($resql)
        {
            if ($this->db->num_rows($resql))
            {
                $obj = $this->db->fetch_object($resql);

                $this->id = $obj->rowid;
								$this->carddav_etag= $obj->carddav_etag;
								$this->carddav_id = $obj->carddav_id;
								
								$this->fk_socid= $obj->fk_socid;
                $this->fk_contactid = $obj->fk_contactid;
                $this->last_modified = $obj->last_modified;

                $this->statut = $obj->statut;
                $this->updateprogress= $obj->updateprogress;
               
                $this->fk_user_carddav = $obj->fk_user_carddav;
                $this->fk_user = $obj->fk_user;
                
                
								return 1;
            }
            $this->db->free($resql);

            return 0;
        }
        else
        {
            $this->error = "Error " . $this->db->lasterror();
            dol_syslog(get_class($this) . "::fetch_from_user " . $this->error,
                                 LOG_ERR);
            return -1;
        }
    }

    /**
     *  Load object in memory from the database
     *
     *  @param	int		$id    Id object
     *  @return int          	<0 if KO, >0 if OK
     */
    function fetch($id)
    {
        global $langs;
        $sql = "SELECT";
        $sql.= " t.rowid,";
        $sql.= " t.carddav_etag,";
        $sql.= " t.carddav_id,";
        $sql.= " t.fk_socid,";
        $sql.= " t.fk_contactid,";
        $sql.= " t.last_modified,";
        $sql.= " t.statut,";
				$sql.= " t.fk_user_carddav,";
        $sql.= " t.fk_user,";
        $sql.= " t.updateprogress";


        $sql.= " FROM " . MAIN_DB_PREFIX . "carddav_link as t";
        $sql.= " WHERE t.rowid = " . $id;

        dol_syslog(get_class($this) . "::fetch sql=" . $sql, LOG_DEBUG);
        $resql = $this->db->query($sql);
        if ($resql)
        {
            if ($this->db->num_rows($resql))
            {
                $obj = $this->db->fetch_object($resql);

                $this->id = $obj->rowid;
								$this->carddav_etag= $obj->carddav_etag;
								$this->carddav_id = $obj->carddav_id;
								
								$this->fk_socid= $obj->fk_socid;
                $this->fk_contactid = $obj->fk_contactid;
                $this->last_modified = $obj->carddav_password;
                
                
                $this->statut = $obj->statut;
                $this->updateprogress= $obj->updateprogress;
                
                $this->fk_user = $obj->fk_user;
                $this->fk_user_carddav = $obj->fk_user_carddav;
            }
            $this->db->free($resql);

            return 1;
        }
        else
        {
            $this->error = "Error " . $this->db->lasterror();
            dol_syslog(get_class($this) . "::fetch " . $this->error, LOG_ERR);
            return -1;
        }
    }

    /**
     *  Update object into database
     *
     *  @param	User	$user        User that modifies
     *  @param  int		$notrigger	 0=launch triggers after, 1=disable triggers
     *  @return int     		   	 <0 if KO, >0 if OK
     */
    function update($user = 0, $notrigger = 0)
    {
        global $conf, $langs;
        $error = 0;

        if (isset($this->carddav_etag))
						$this->carddav_etag = trim($this->carddav_etag);
				if (isset($this->carddav_id))
						$this->carddav_id = trim($this->carddav_id);
						
        if (isset($this->fk_socid))
						$this->fk_socid = (int)$this->fk_socid;
        if (isset($this->fk_contactid))
						$this->fk_contactid = (int)$this->fk_contactid;
        if (isset($this->last_modified))
						$this->last_modified = trim($this->last_modified);
        if (isset($this->statut))
						$this->statut = (int)$this->statut;
						
        if (isset($this->fk_user)) 
						$this->fk_user = trim($this->fk_user);
				if (isset($this->fk_user_carddav)) 
						$this->fk_user_carddav = trim($this->fk_user_carddav);



        $sql = "UPDATE " . MAIN_DB_PREFIX . "carddav_link SET";

        if(isset($this->fk_socid) && $this->fk_socid > 0 )
					$sql.= " fk_socid='" . $this->db->escape($this->fk_socid) . "',";
					
				if(isset($this->fk_contactid) && $this->fk_contactid > 0 )
					$sql.= " fk_contactid='" . $this->db->escape($this->fk_contactid) . "',";
					
					
        $sql.= " last_modified= NOW(),";
        $sql.= " statut=" . (isset($this->statut) ? "'" . $this->statut . "'" : "0") . ", ";
        $sql.= " fk_user=" . $user->id . "";
        $sql.= " WHERE rowid='" . $this->id."'  ";

// echo $sql; 
        $this->db->begin();

        dol_syslog(get_class($this) . "::update sql=" . $sql, LOG_DEBUG);
        $resql = $this->db->query($sql);
        if (!$resql)
        {
            $error++;
            $this->errors[] = "Error " . $this->db->lasterror();
        }

        if (!$error)
        {
            if (!$notrigger)
            {
            }
        }


        if ($error)
        {
            foreach ($this->errors as $errmsg)
            {
                $this->error.=($this->error ? ', ' . $errmsg : $errmsg);
            }
            $this->db->rollback();
            return -1 * $error;
        }
        else
        {
            $this->db->commit();
            return 1;
        }
    }

   /**
     *  Update object into database in current progess update ; fix this row 
     *
     *  @param	User	$user        User that modifies
     *  @param  int		$notrigger	 0=launch triggers after, 1=disable triggers
     *  @return int     		   	 <0 if KO, >0 if OK
     */
    function UpdateStart($user = 0, $notrigger = 0)
    {
        global $conf, $langs;
        $error = 0;

        $sql = "UPDATE " . MAIN_DB_PREFIX . "carddav_link SET";
        $sql.= " updateprogress = 1 ";
        $sql.= " WHERE rowid='" . $this->id."'  ";

        $this->db->begin();

        dol_syslog(get_class($this) . "::UpdateStart sql=" . $sql, LOG_DEBUG);
        $resql = $this->db->query($sql);
        if (!$resql)
        {
            $error++;
            $this->errors[] = "Error " . $this->db->lasterror();
        }

        if (!$error)
        {
            if (!$notrigger)
            {
            }
        }


        if ($error)
        {
            foreach ($this->errors as $errmsg)
            {
                $this->error.=($this->error ? ', ' . $errmsg : $errmsg);
            }
            $this->db->rollback();
            return -1 * $error;
        }
        else
        {
            $this->db->commit();
            return 1;
        }
    }
    
   /**
     *  Update object into database in current progess update ; fix this row 
     *
     *  @param	User	$user        User that modifies
     *  @param  int		$notrigger	 0=launch triggers after, 1=disable triggers
     *  @return int     		   	 <0 if KO, >0 if OK
     */
    function UpdateEnd($user = 0, $notrigger = 0)
    {
        global $conf, $langs;
        $error = 0;

        $sql = "UPDATE " . MAIN_DB_PREFIX . "carddav_link SET";
        $sql.= " last_modified= NOW(),";
        $sql.= " updateprogress=0 ";
        $sql.= " WHERE rowid='" . $this->id."'  ";

        $this->db->begin();

        dol_syslog(get_class($this) . "::UpdateEnd sql=" . $sql, LOG_DEBUG);
        $resql = $this->db->query($sql);
        if (!$resql)
        {
            $error++;
            $this->errors[] = "Error " . $this->db->lasterror();
        }

        if (!$error)
        {
            if (!$notrigger)
            {
            }
        }


        if ($error)
        {
            foreach ($this->errors as $errmsg)
            {
                $this->error.=($this->error ? ', ' . $errmsg : $errmsg);
            }
            $this->db->rollback();
            return -1 * $error;
        }
        else
        {
            $this->db->commit();
            return 1;
        }
    }
    
    /**
     *  Delete object in database
     *
     * 	@param  User	$user        User that deletes
     *  @param  int		$notrigger	 0=launch triggers after, 1=disable triggers
     *  @return	int					 <0 if KO, >0 if OK
     */
    function delete($user, $notrigger = 0)
    {
        global $conf, $langs;
        $error = 0;

        $this->db->begin();

        if (!$error)
        {
            if (!$notrigger)
            {
            }
        }

        if (!$error)
        {
            $sql = "DELETE FROM " . MAIN_DB_PREFIX . "carddav_link";
            $sql.= " WHERE rowid=" . $this->id;

            dol_syslog(get_class($this) . "::delete sql=" . $sql);
            $resql = $this->db->query($sql);
            if (!$resql)
            {
                $error++;
                $this->errors[] = "Error " . $this->db->lasterror();
            }
        }

        if ($error)
        {
            foreach ($this->errors as $errmsg)
            {
                dol_syslog(get_class($this) . "::delete " . $errmsg, LOG_ERR);
                $this->error.=($this->error ? ', ' . $errmsg : $errmsg);
            }
            $this->db->rollback();
            return -1 * $error;
        }
        else
        {
            $this->db->commit();
            return 1;
        }
    }

    /**
     * 	Load an object from its id and create a new one in database
     *
     * 	@param	int		$fromid     Id of object to clone
     * 	@return	int					New id of clone
     */
    function createFromClone($fromid)
    {
        global $user, $langs;

        $error = 0;

        $object = new self($this->db);

        $this->db->begin();


        $object->fetch($fromid);
        $object->id = 0;
        $object->statut = 0;


        $result = $object->create($user);


        if ($result < 0)
        {
            $this->error = $object->error;
            $error++;
        }

        if (!$error)
        {
            
        }

        if (!$error)
        {
            $this->db->commit();
            return $object->id;
        }
        else
        {
            $this->db->rollback();
            return -1;
        }
    }

    /**
     * 	Initialise object with example values
     * 	Id must be 0 if object instance is a specimen
     *
     * 	@return	void
     */
    function initAsSpecimen(){
        $this->id = 0;
				$this->carddav_etag = 1;
        $this->fk_socid = '';
        $this->fk_contactid = '';
        $this->last_modified = '';
        $this->status= 1;
        $this->fk_user = 0;
        $this->fk_user_carddav = 0;
    }


}

?>
