<?php
/* 
 * Copyright (C) 2014 Oscim       <oscim@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 *  \file       
 *  \ingroup    
 *  \brief      This file is an example for a CRUD class file (Create/Read/Update/Delete)
 * 			
 */


/**
 * 	Put here description of your class
 */
class Usercarddavconfig
{

    public $db;       //!< To store db handler
    /**
			@var 
    */
    public $error;       //!< To return error code (or message)
    /**
			@var 
    */
    public $errors = array();    //!< To return several error codes (or messages)
    /**
			@var 
    */
    public $id;
    /**
			@var 
    */
    public $carddav_login;
    /**
			@var 
    */
    public $carddav_password;
    /**
			@var 
    */
    public $carddav_host;
    /**
			@var 
    */
    public $carddav_port;
    /**
			@var 
    */
    public $carddav_ssl;
    /**
			@var 
    */
    public $carddav_ssl_novalidate_cert;
    /**
			@var 
    */
    public $fk_user;
    /**
			@var 
    */
    public $number = 1;
    /**
			@var carddav_type integer 
    */
    public $carddav_type = 'Generic';
    /**
			@var 
    */
    public $frequency;
    
    /**
     *  Constructor
     *
     *  @param	DoliDb		$db      Database handler
     */
    function __construct($db)
    {
        $this->db = $db;
        return 1;
    }

    /**
     *  Create object into database
     *
     *  @param	User	$user        User that creates
     *  @param  int		$notrigger   0=launch triggers after, 1=disable triggers
     *  @return int      		   	 <0 if KO, Id of created object if OK
     */
    function create($user, $notrigger = 0)
    {
        global $conf, $langs;
        $error = 0;

        if (isset($this->carddav_login))
					$this->carddav_login = trim($this->carddav_login);
        if (isset($this->carddav_password))
					$this->carddav_password = trim($this->carddav_password);
        if (isset($this->carddav_host))
					$this->carddav_host = trim($this->carddav_host);
				if (isset($this->carddav_port))
					$this->carddav_port = trim($this->carddav_port);
        if (isset($this->carddav_type))
					$this->carddav_type = $this->carddav_type;
				if (isset($this->carddav_path))
					$this->carddav_path = trim($this->carddav_path);
        if (isset($this->carddav_ssl))
					$this->carddav_ssl = trim($this->carddav_ssl);
        if (isset($this->carddav_ssl_novalidate_cert))
					$this->carddav_ssl_novalidate_cert = trim($this->carddav_ssl_novalidate_cert);
        if (isset($this->fk_user)) 
					$this->fk_user = trim($this->fk_user);
				if (isset($this->fk_user)) 
					$this->last_modified = trim($this->fk_user);
				if (isset($this->frequency)) 
					$this->frequency = $this->frequency;


        $sql = "INSERT INTO " . MAIN_DB_PREFIX . "user_carddav(";
				$sql.= "number,";
				$sql.= "carddav_type,";
        $sql.= "carddav_login,";
        $sql.= "carddav_password,";
        $sql.= "carddav_host,";
        $sql.= "carddav_port,";
        $sql.= "carddav_path,";
        $sql.= "carddav_ssl,";
        $sql.= "carddav_ssl_novalidate_cert,";
        $sql.= "fk_user, ";
        $sql.= "frequency,";
        $sql.= "last_modified";
        $sql.= ") VALUES (";
        $sql.= " " . (!isset($this->number) ? '1' : "'" . (int)$this->number . "'") . ",";
        $sql.= " " . (!isset($this->carddav_type) ? 'Generic' : "'" . $this->db->escape($this->carddav_type) . "'") . ",";
        $sql.= " " . (!isset($this->carddav_login) ? 'NULL' : "'" . $this->db->escape($this->carddav_login) . "'") . ",";
        $sql.= " " . (!isset($this->carddav_password) ? 'NULL' : "'" . $this->db->escape($this->carddav_password) . "'") . ",";
        $sql.= " " . (!isset($this->carddav_host) ? 'NULL' : "'" . $this->db->escape($this->carddav_host) . "'") . ",";
        $sql.= " " . (!isset($this->carddav_port) ? 'NULL' : "'" . $this->db->escape($this->carddav_port) . "'") . ",";
				$sql.= " " . (!isset($this->carddav_path) ? 'NULL' : "'" . $this->db->escape($this->carddav_path) . "'") . ",";
        $sql.= " " . (!isset($this->carddav_ssl) ? 'NULL' : "'" . $this->db->escape($this->carddav_ssl) . "'") . ",";
        $sql.= " " . (!isset($this->carddav_ssl_novalidate_cert) ? 'NULL' : "'" . $this->db->escape($this->carddav_ssl_novalidate_cert) . "'") . ",";
        $sql.= " " . (!isset($this->fk_user) ? 'NULL' : "'" . $this->fk_user . "'") . ", ";
        $sql.= " " . (!isset($this->frequency) ? '3600' : "'" . $this->frequency . "'") . ", ";
        $sql.= " NOW()";
        $sql.= ")";

        $this->db->begin();

        dol_syslog(get_class($this) . "::create sql=" . $sql, LOG_DEBUG);
        $resql = $this->db->query($sql);
        if (!$resql)
        {
            $error++;
            $this->errors[] = "Error " . $this->db->lasterror();
        }

        if (!$error)
        {
            $this->id = $this->db->last_insert_id(MAIN_DB_PREFIX . "user_carddav");

            if (!$notrigger)
            {
            }
        }


        if ($error)
        {
            foreach ($this->errors as $errmsg)
            {
                dol_syslog(get_class($this) . "::create " . $errmsg, LOG_ERR);
                $this->error.=($this->error ? ', ' . $errmsg : $errmsg);
            }
            $this->db->rollback();
            return -1 * $error;
        }
        else
        {
            $this->db->commit();
            return $this->id;
        }
    }

    /**
     *  Load object in memory from the database
     *
     *  @param	int		$fk_user    id of user to load
     *  @param int $number indice 
     *  @return int          	<0 if KO, >0 if OK
     */
    function fetch_from_user($fk_user, $number=1)
    {
        global $langs;
        $sql = "SELECT";
        $sql.= " t.rowid,";
				$sql.= " t.number,";
				$sql.= " t.carddav_type,";
        $sql.= " t.carddav_login,";
        $sql.= " t.carddav_password,";
        $sql.= " t.carddav_host,";
        $sql.= " t.carddav_port,";
        $sql.= " t.carddav_path,";
        $sql.= " t.carddav_ssl,";
        $sql.= " t.carddav_ssl_novalidate_cert,";
        $sql.= " t.fk_user,";
        $sql.= " t.frequency,";
        $sql.= " t.last_modified";


        $sql.= " FROM " . MAIN_DB_PREFIX . "user_carddav as t";
        $sql.= " WHERE t.fk_user = '" . (int)$fk_user."' AND t.number= '".(int)$number."' ";

        $resql = $this->db->query($sql);
        if ($resql)
        {
            if ($this->db->num_rows($resql))
            {
                $obj = $this->db->fetch_object($resql);

                $this->id = $obj->rowid;
								$this->number= $obj->number;
								
								$this->carddav_type= $obj->carddav_type;
                $this->carddav_login = $obj->carddav_login;
                $this->carddav_password = $obj->carddav_password;
                $this->carddav_host = $obj->carddav_host;
                $this->carddav_port = $obj->carddav_port;
                $this->carddav_path = $obj->carddav_path;
                $this->carddav_ssl = $obj->carddav_ssl;
                $this->carddav_ssl_novalidate_cert = $obj->carddav_ssl_novalidate_cert;
                $this->fk_user = $obj->fk_user;
								$this->last_modified= $obj->last_modified;
                $this->frequency = $obj->frequency;
            }
            $this->db->free($resql);

            return 1;
        }
        else
        {
            $this->error = "Error " . $this->db->lasterror();
            dol_syslog(get_class($this) . "::fetch_from_user " . $this->error,
                                 LOG_ERR);
            return -1;
        }
    }

    /**
     *  Load object in memory from the database
     *
     *  @param	int		$id    Id object
     *  @return int          	<0 if KO, >0 if OK
     */
    function fetch($id)
    {
        global $langs;
        $sql = "SELECT";
        $sql.= " t.rowid,";
        $sql.= " t.carddav_type,";
        $sql.= " t.number,";
        $sql.= " t.carddav_login,";
        $sql.= " t.carddav_password,";
        $sql.= " t.carddav_host,";
        $sql.= " t.carddav_port,";
        $sql.= " t.carddav_path,";
        $sql.= " t.carddav_ssl,";
        $sql.= " t.carddav_ssl_novalidate_cert,";
        $sql.= " t.fk_user,";
				$sql.= " t.frequency,";
        $sql.= " t.last_modified";


        $sql.= " FROM " . MAIN_DB_PREFIX . "user_carddav as t";
        $sql.= " WHERE t.rowid = " . $id;

        dol_syslog(get_class($this) . "::fetch sql=" . $sql, LOG_DEBUG);
        $resql = $this->db->query($sql);
        if ($resql)
        {
            if ($this->db->num_rows($resql))
            {
                $obj = $this->db->fetch_object($resql);

                $this->id = $obj->rowid;

                $this->carddav_login = $obj->carddav_login;
                $this->carddav_password = $obj->carddav_password;
                $this->carddav_host = $obj->carddav_host;
                $this->carddav_port = $obj->carddav_port;
                $this->carddav_type = $obj->carddav_type;
                $this->carddav_path = $obj->carddav_path;
                $this->number = $obj->number;
                $this->carddav_ssl = $obj->carddav_ssl;
                $this->carddav_ssl_novalidate_cert = $obj->carddav_ssl_novalidate_cert;
                $this->fk_user = $obj->fk_user;
                $this->last_modified= $obj->last_modified;
                $this->frequency = $obj->frequency;
            }
            $this->db->free($resql);

            return 1;
        }
        else
        {
            $this->error = "Error " . $this->db->lasterror();
            dol_syslog(get_class($this) . "::fetch " . $this->error, LOG_ERR);
            return -1;
        }
    }

    /**
     *  Update object into database
     *
     *  @param	User	$user        User that modifies
     *  @param  int		$notrigger	 0=launch triggers after, 1=disable triggers
     *  @return int     		   	 <0 if KO, >0 if OK
     */
    function update($user = 0, $notrigger = 0)
    {
        global $conf, $langs;
        $error = 0;

        if (isset($this->carddav_login))
					$this->carddav_login = trim($this->carddav_login);
        if (isset($this->carddav_password))
					$this->carddav_password = trim($this->carddav_password);
				if (isset($this->carddav_port))
					$this->carddav_port = trim($this->carddav_port);
        if (isset($this->carddav_host))
					$this->carddav_host = trim($this->carddav_host);
				if (isset($this->carddav_path))
					$this->carddav_path = trim($this->carddav_path);
        if (isset($this->carddav_type))
					$this->carddav_type = $this->carddav_type;
        if (isset($this->carddav_ssl))
					$this->carddav_ssl = trim($this->carddav_ssl);
        if (isset($this->carddav_ssl_novalidate_cert))
					$this->carddav_ssl_novalidate_cert = trim($this->carddav_ssl_novalidate_cert);
        if (isset($this->fk_user)) 
					$this->fk_user = trim($this->fk_user);
				if (isset($this->fk_user)) 
					$this->last_modified = trim($this->fk_user);
				if (isset($this->frequency)) 
					$this->frequency = $this->frequency;



        $sql = "UPDATE " . MAIN_DB_PREFIX . "user_carddav SET";
				
				$sql.= " carddav_type=" . (isset($this->carddav_type) ? "'" . $this->db->escape($this->carddav_type) . "'" : "Generic") . ",";
        $sql.= " carddav_login=" . (isset($this->carddav_login) ? "'" . $this->db->escape($this->carddav_login) . "'" : "null") . ",";
        $sql.= " carddav_password=" . (isset($this->carddav_password) ? "'" . $this->db->escape($this->carddav_password) . "'" : "null") . ",";
        $sql.= " carddav_host=" . (isset($this->carddav_host) ? "'" . $this->db->escape($this->carddav_host) . "'" : "null") . ",";
        $sql.= " carddav_port=" . (isset($this->carddav_port) ? "'" . $this->db->escape($this->carddav_port) . "'" : "null") . ",";
        $sql.= " carddav_path=" . (isset($this->carddav_path) ? "'" . $this->db->escape($this->carddav_path) . "'" : "null") . ",";
        $sql.= " carddav_ssl=" . (isset($this->carddav_ssl) ? "'" . $this->db->escape($this->carddav_ssl) . "'" : "null") . ",";
        $sql.= " carddav_ssl_novalidate_cert=" . (isset($this->carddav_ssl_novalidate_cert)  ? "'" . $this->db->escape($this->carddav_ssl_novalidate_cert) . "'" : "null") . ",";
        $sql.= " fk_user=" . (isset($this->fk_user) ? $this->fk_user : "null") . ",";
				$sql.= " last_modified= NOW(), ";
				$sql.= " frequency=" . (isset($this->frequency) ? $this->frequency : "3600") . " ";
        $sql.= " WHERE rowid='" . $this->id."' ";
// echo $sql; 
// exit;
        $this->db->begin();

        dol_syslog(get_class($this) . "::update sql=" . $sql, LOG_DEBUG);
        $resql = $this->db->query($sql);
        if (!$resql)
        {
            $error++;
            $this->errors[] = "Error " . $this->db->lasterror();
        }

        if (!$error)
        {
            if (!$notrigger)
            {
            }
        }


        if ($error)
        {
            foreach ($this->errors as $errmsg)
            {
                $this->error.=($this->error ? ', ' . $errmsg : $errmsg);
            }
            $this->db->rollback();
            return -1 * $error;
        }
        else
        {
            $this->db->commit();
            return 1;
        }
    }

    /**
     *  Delete object in database
     *
     * 	@param  User	$user        User that deletes
     *  @param  int		$notrigger	 0=launch triggers after, 1=disable triggers
     *  @return	int					 <0 if KO, >0 if OK
     */
    function delete($user, $notrigger = 0)
    {
        global $conf, $langs;
        $error = 0;

        $this->db->begin();

        if (!$error)
        {
            if (!$notrigger)
            {
            }
        }

        if (!$error)
        {
            $sql = "DELETE FROM " . MAIN_DB_PREFIX . "user_carddav";
            $sql.= " WHERE rowid=" . $this->id;

            dol_syslog(get_class($this) . "::delete sql=" . $sql);
            $resql = $this->db->query($sql);
            if (!$resql)
            {
                $error++;
                $this->errors[] = "Error " . $this->db->lasterror();
            }
        }

        if ($error)
        {
            foreach ($this->errors as $errmsg)
            {
                dol_syslog(get_class($this) . "::delete " . $errmsg, LOG_ERR);
                $this->error.=($this->error ? ', ' . $errmsg : $errmsg);
            }
            $this->db->rollback();
            return -1 * $error;
        }
        else
        {
            $this->db->commit();
            return 1;
        }
    }

    /**
     * 	Load an object from its id and create a new one in database
     *
     * 	@param	int		$fromid     Id of object to clone
     * 	@return	int					New id of clone
     */
    function createFromClone($fromid)
    {
        global $user, $langs;

        $error = 0;

        $object = new Usermailboxconfig($this->db);

        $this->db->begin();


        $object->fetch($fromid);
        $object->id = 0;
        $object->statut = 0;


        $result = $object->create($user);


        if ($result < 0)
        {
            $this->error = $object->error;
            $error++;
        }

        if (!$error)
        {
            
        }

        if (!$error)
        {
            $this->db->commit();
            return $object->id;
        }
        else
        {
            $this->db->rollback();
            return -1;
        }
    }

    /**
     * 	Initialise object with example values
     * 	Id must be 0 if object instance is a specimen
     *
     * 	@return	void
     */
    function initAsSpecimen(){
        $this->id = 0;
				$this->number = 1;
        $this->carddav_login = '';
        $this->carddav_password = '';
        $this->carddav_host = '';
        $this->carddav_port = '';
        $this->carddav_path = '';
        $this->carddav_type = 'Generic';
        $this->carddav_ssl = 0;
        $this->carddav_ssl_novalidate_cert = 0;
        $this->fk_user = 0;
        $this->last_modified= 0;
        $this->frequency= 3600;
    }

    /**
    */
		public function GetServer(){

			dol_include_once('carddav/core/modules/carddavmods/'. strtolower($this->carddav_type).'.class.php');
			$cl = $this->carddav_type.'CardDav'; 

			$obj = new $cl($this); 
			$path = $obj->GenerateDir(); 

			return $path;
		}

		/**
			@brief return string type name 
		*/
		public function GetCardDavType($typ=-1){
			global $conf; 


			if((string)$typ == '0'){
				clearstatcache();
				foreach ($conf->file->dol_document_root as $dirroot){
				
					$dir = $dirroot . "/carddav/core/modules/carddavmods/";

					if (is_dir($dir))
					{
						$handle = opendir($dir);
						if (is_resource($handle))
						{
							$var=true;

							while (($file = readdir($handle))!==false)
							{

								if ( substr($file, dol_strlen($file)-3, 3) == 'php')
								{
									$file = substr($file, 0, dol_strlen($file)-4);

									require_once($dir . $file.".php");

									dol_include_once( $dir . $file.".php" );

									$cl = ucwords(substr($file , 0, -6).'CardDav'); 
									$module = new $cl($this); 

									$type[] = $module;
								}
							}
							closedir($handle);
						}
					}
				}
			}
			elseif((string)$typ =='-1'){
				dol_include_once('carddav/core/modules/carddavmods/'. strtolower($this->carddav_type).'.class.php');
				$cl = $this->carddav_type.'CardDav'; 
				
				$obj = new $cl($this); 
				$type = $obj->GetTitle(); 
			}
			else{

				dol_include_once('carddav/core/modules/carddavmods/'. strtolower($typ).'.class.php');
				$cl = $typ.'CardDav'; 
				
				$obj = new $cl($this); 
				$type = $obj->GetTitle(); 
				
			}
			
			return $type;
		}
		

		
		
		/**
			@fn  GetFileNameTmp(User $user, $user_carddav )
			@brief 
			@param User $user
			@param 
			@return string file path 
		*/
		public function GetFileNameTmp(User $user, $user_carddav ){
			global $conf; 
			
			$file_name = 'carddav_'.$user->id.'-'.$user_carddav->carddav_type.'-'.$user_carddav->number.'.xml'; 
			$path = $conf->carddav->dir_output;
			
			return $path.'/'.$file_name;
		}
}

?>
