<?php
/* Copyright (C) 2005      Rodolphe Quiedeville <rodolphe@quiedeville.org>
 * Copyright (C) 2005-2008 Laurent Destailleur  <eldy@users.sourceforge.org>
 * Copyright (C) 2012		 Oscim					       <oscim@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 *	\defgroup   calling      Module calling
 *	\brief      Module pour gerer l'appel automatique
 */

/**
 *	\file       htdocs/includes/modules/modCalling.class.php
 *	\ingroup    calling
 *	\brief      Fichier de description et activation du module de click to Dial
 *	\author     Oscim <mail oscim@users.sourceforge.net>
 *	\version    $Id: modCalling.php,v 1.56 2012/06/10 15:28:01 oscim Exp $
 */

include_once(DOL_DOCUMENT_ROOT ."/core/modules/DolibarrModules.class.php");


/**
 \class      modCalling
 \brief      Classe de description et activation du module de Click to Dial
 */

class modcarddav extends DolibarrModules
{

	/**
	 *   \brief      Constructeur. Definit les noms, constantes et boites
	 *   \param      DB      handler d'acces base
	 */
	function modcarddav($DB)
	{
		$this->db = $DB ;
		$this->numero = 98365 ;

		$this->family = "technic";
		// Module label (no space allowed), used if translation string 'ModuleXXXName' not found (where XXX is value of numeric property 'numero' of module)
		$this->name = preg_replace('/^mod/i','',get_class($this));
		$this->description = "CardDav";

		$this->version = '0.8';

		$this->const_name = 'MAIN_MODULE_'.strtoupper($this->name);
		$this->special = 1;
		$this->picto='carddav.png@carddav';

		// Data directories to create when module is enabled
		$this->dirs = array("/carddav");
		
		$this->phpmin = array(5, 3);

		$this->need_dolibarr_version = array(3, 5);
		
		// Dependencies
		$this->depends = array('modframework', 'modSociete' );
		$this->requiredby = array();

		$this->conflictwith = array();
		$this->langfiles = array("carddav@carddav","users");

		// Defined all module parts (triggers, login, substitutions, menus, etc...) (0=disable,1=enable)
		$this->module_parts = array('triggers' => 1);
   
		// tabs
		$this->tabs = array(
						'user:+TabCardDavConf:TabCardDavConf:carddav:@carddav:/carddav/tabs/user.carddavconfig.php?id=__ID__',
				    'thirdparty:+TabCardDav:TabCardDav:carddav:@carddav:/carddav/tabs/societe.carddav.php?socid=__ID__',
				    'contact:+TabCardDav:TabCardDav:carddav:@carddav:/carddav/tabs/contact.carddav.php?id=__ID__',
				    );

		// Config pages
		$this->config_page_url = array(dol_buildpath("/carddav/admin/index.php", 2 ) );

		// Constantes
		$this->const = array();

		global $langs; 
		
		$r=0;
		$this->const[$r][0] = "CARDDAV_ADDON";
		$this->const[$r][1] = "chaine";
		$this->const[$r][2] = "";
		$this->const[$r][3] = "Carddav addon active";
		$this->const[$r][4] = 1;
		$this->const[$r][5] = 'current';
		$this->const[$r][6] = 1; // supprime la constante à la désactivation du module

		$r++; 
		$this->const[$r][0] = "CARDDAV_PUT_SERVER";
		$this->const[$r][1] = "int";
		$this->const[$r][2] = "0";
		$this->const[$r][3] = "carddavPutOnServerDetail";
		$this->const[$r][4] = 1;
		$this->const[$r][5] = 'current';
		$this->const[$r][6] = 1; // supprime la constante à la désactivation du module

		$r++; 
		$this->const[$r][0] = "CARDDAV_CREATE_SERVER";
		$this->const[$r][1] = "int";
		$this->const[$r][2] = "0";
		$this->const[$r][3] = "carddavCreateOnServerDetail";
		$this->const[$r][4] = 1;
		$this->const[$r][5] = 'current';
		$this->const[$r][6] = 1; // supprime la constante à la désactivation du module
		
		$r++; 
		$this->const[$r][0] = "CARDDAV_DELETE_SERVER";
		$this->const[$r][1] = "int";
		$this->const[$r][2] = "0";
		$this->const[$r][3] = "carddavDeleteOnServerDetail";
		$this->const[$r][4] = 1;
		$this->const[$r][5] = 'current';
		$this->const[$r][6] = 1; // supprime la constante à la désactivation du module
		
		$r++; 
		$this->const[$r][0] = "CARDDAV_FICHE_TYPE";
		$this->const[$r][1] = "int";
		$this->const[$r][2] = "0";
		$this->const[$r][3] = "carddavFicheInDolDetail";
		$this->const[$r][4] = 1;
		$this->const[$r][5] = 'current';
		$this->const[$r][6] = 1; // supprime la constante à la désactivation du module
		
		
		// Boxes
		$this->boxes = array();

		// Permissions
		$this->rights = array();
		$this->rights_class = 'carddav';


		// Main menu entries
		$this->menu = array();			// List of menus to add
// 		$r=0;
// 		$this->menu[$r]=array('fk_menu'=>0,
// 													'type'=>'top',
// 													'titre'=>'Ed',
// 													'mainmenu'=>'carddav',
// 													'leftmenu'=>'1',		// Use 1 if you also want to add left menu entries using this descriptor.
// 													'url'=>'/carddav/index.php',
// 													'langs'=>'carddav',
// 													'position'=>100,
// 													'perms'=>'$user->rights->carddav->carddavaction->read',
// 													'enabled'=>'$conf->carddav->enabled',
// 													'target'=>'',
// 													'user'=>2);
		$r++;
	}

    /**
     *      \brief      Function called when module is enabled.
     *                  The init function add constants, boxes, permissions and menus (defined in constructor) into Dolibarr database.
     *                  It also creates data directories.
     *      \return     int             1 if OK, 0 if KO
     */
		function init()
		{
			global $conf;

			$sql = array();

		$this->load_tables();

			return $this->_init($sql);
		}

    /**
     *      \brief      Function called when module is disabled.
     *                  Remove from database constants, boxes and permissions from Dolibarr database.
     *                  Data directories are not deleted.
     *      \return     int             1 if OK, 0 if KO
     */
		function remove($options = '')
		{
		global $conf;
		
		$sql = array(
		);

		return $this->_remove($sql, $options);
		}


    /**
     * 		Create tables, keys and data required by module
     * 		Files llx_table1.sql, llx_table1.key.sql llx_data.sql with create table, create keys
     * 		and create data commands must be stored in directory /mymodule/sql/
     * 		This function is called by this->init
     *
     * 		@return		int		<=0 if KO, >0 if OK
     */
    function load_tables() {
        return $this->_load_tables(dol_buildpath('/carddav/sql/', 0 ));
    }

}
?>
