<?php
/* Copyright (C) 2005-2011 Laurent Destailleur  <eldy@users.sourceforge.net>
 * Copyright (C) 2005-2011 Regis Houssin        <regis.houssin@capnetworks.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 *  \file       htdocs/core/triggers/
 *  \ingroup    core
 *  \brief      Fichier de demo de personalisation des actions du workflow
 *  \remarks    Son propre fichier d'actions peut etre cree par recopie de celui-ci:
 *              - Le nom du fichier doit etre: interface_99_modMymodule_Mytrigger.class.php
 *				                           ou: interface_99_all_Mytrigger.class.php
 *              - Le fichier doit rester stocke dans core/triggers
 *              - Le nom de la classe doit etre InterfaceMytrigger
 *              - Le nom de la propriete name doit etre Mytrigger
 */


/**
 *  Class of triggers for demo module
 */
class Interfacecarddavsynchro
{
    var $db;
    
    /**
     *   Constructor
     *
     *   @param		DoliDB		$db      Database handler
     */
    function __construct($db)
    {
        $this->db = $db;
    
        $this->name = preg_replace('/^Interface/i','',get_class($this));
        $this->family = "demo";
        $this->description = "Triggers of this module are empty functions. They have no effect. They are provided for tutorial purpose only.";
        $this->version = 'dolibarr';            // 'development', 'experimental', 'dolibarr' or version
        $this->picto = 'technic';
    }
    
    
    /**
     *   Return name of trigger file
     *
     *   @return     string      Name of trigger file
     */
    function getName()
    {
        return $this->name;
    }
    
    /**
     *   Return description of trigger file
     *
     *   @return     string      Description of trigger file
     */
    function getDesc()
    {
        return $this->description;
    }

    /**
     *   Return version of trigger file
     *
     *   @return     string      Version of trigger file
     */
    function getVersion()
    {
        global $langs;
        $langs->load("admin");

        if ($this->version == 'development') return $langs->trans("Development");
        elseif ($this->version == 'experimental') return $langs->trans("Experimental");
        elseif ($this->version == 'dolibarr') return DOL_VERSION;
        elseif ($this->version) return $this->version;
        else return $langs->trans("Unknown");
    }
    
    /**
     *      Function called when a Dolibarrr business event is done.
     *      All functions "run_trigger" are triggered if file is inside directory htdocs/core/triggers
     *
     *      @param	string		$action		Event action code
     *      @param  Object		$object     Object
     *      @param  User		$user       Object user
     *      @param  Translate	$langs      Object langs
     *      @param  conf		$conf       Object conf
     *      @return int         			<0 if KO, 0 if no triggered ran, >0 if OK
     */
	function run_trigger($action,$object,$user,$langs,$conf)
    {
    
				global $db; 
				
				// block no process triggers 
				if(defined('CARDAV_NOTRIGGERS') || CARDAV_NOTRIGGERS == false )
					return 0; 
					
				// No process Desactived in Global Configuration 
				if( $conf->global->CARDDAV_PUT_SERVER == 0 )
					return 0; 

        // Put here code you want to execute when a Dolibarr business events occurs.
        // Data and type of action are stored into $object and $action
    
				dol_include_once('carddav/core/lib/carddav.lib.php');

				dol_include_once('carddav/class/carddav.class.php');
				dol_include_once('carddav/class/Synchro.class.php');
				dol_include_once('carddav/class/DolCardDav.class.php');
				dol_include_once('carddav/class/user.carddav.class.php');
				dol_include_once('carddav/class/societe.carddav.class.php');

				dol_include_once('carddav/class/vcardphp-1.1.2/vcard.php');
				dol_include_once('carddav/class/vcardphp-1.1.2/vbook.php');

				$user_carddav = new Usercarddavconfig($db);
				$DolCardDav = new DolCardDav($db); 
				$result = array();

				$sql = "SELECT * ";
				$sql.= " FROM ".MAIN_DB_PREFIX."user_carddav";
				$sql.= " WHERE  1 ";
				$sql.= " AND fk_user = '".$user->id."' ";
				$resql = $db->query($sql);
				if ($resql)
				{
					$num = $db->num_rows($resql);

					$i = 0;
					$var=true;
					while ($i < $num)
					{
						$objp = $db->fetch_object($resql);
						$user_carddav->fetch($objp->rowid);
						$carddav = new carddav_backend( $user_carddav->GetServer() );
						
						$carddav->set_auth($user_carddav->carddav_login, $user_carddav->carddav_password);
						$carddav->check_connection();
						
						
						
						
						// Members
						if ($action == 'CONTACT_CREATE'  || $action == 'CONTACT_MODIFY' ){
		
							$res = $DolCardDav->fetch_from($user->id, '', 0, $object->id ,'', false, $objp->rowid); 
							
							
							$vcard = Synchro::DolForServer($object);
							
							// put in carddav server 
							if (
										$action == 'CONTACT_CREATE' && 
										(  
												( ( $res <= 0  || $DolCardDav->id <=0 ) && $conf->global->CARDDAV_CREATE_SERVER == 1 ) 
												|| ($res > 0 && $DolCardDav->updateprogress == 1) 
										) 
									)
								$vcard_id = $carddav->add($vcard);
							elseif ($action == 'CONTACT_MODIFY'  
// 									&& $DolCardDav->updateprogress == 0  // no in current process 
									&& $DolCardDav->statut == 1 // no for statut off 
								)
								$vcard_id = $carddav->update($vcard, $DolCardDav->carddav_id);


							$DolCardDav->carddav_id = trim($vcard_id);
							$DolCardDav->fk_contactid = (int)$object->id;
							

							if($res > 0 ){
								$DolCardDav->UpdateStart( $user ); 
								$DolCardDav->update( $user ); 
							}
							else{
								$DolCardDav->statut = (int)1;
								$DolCardDav->fk_user = (int)$user->id;
								$DolCardDav->fk_user_carddav = (int)$objp->rowid;

								$DolCardDav->create( $user ); 
							}
								
								
							$DolCardDav->UpdateEnd( $user ); 
						}
						elseif ($action == 'CONTACT_DELETE' && $conf->global->CARDDAV_DELETE_SERVER == 1){
							
							$res = $DolCardDav->fetch_from($user->id, '', 0, $object->id ,'', false, $objp->rowid); 

							if($res >  0)
								if( ! $carddav->delete($DolCardDav->carddav_id) )
									return false;
						}
						
						
						
						
						
						elseif ($action == 'COMPANY_CREATE' || $action == 'COMPANY_MODIFY'){

							$res = $DolCardDav->fetch_from($user->id, '', $object->id, 0  , '', false, $objp->rowid); 
							
							$vcard = Synchro::DolForServer($object);
							// put in carddav server 
								if (
											$action == 'COMPANY_CREATE' && 
											(  
													( ( $res <= 0  || $DolCardDav->id <=0 ) && $conf->global->CARDDAV_CREATE_SERVER == 1 )
													|| ($res > 0 && $DolCardDav->updateprogress == 0) 
											) 
										)
									$vcard_id = $carddav->add($vcard);
								elseif ($action == 'COMPANY_MODIFY'  
										//&& $DolCardDav->updateprogress == 0 // no in current process 
										&& $DolCardDav->statut == 1 // no for statut off 
								)
									$vcard_id = $carddav->update($vcard, $DolCardDav->carddav_id);


								$DolCardDav->carddav_id = trim($vcard_id);
								$DolCardDav->fk_socid = (int)$object->id;

								if($res > 0  && $DolCardDav->updateprogress == 0 ){
									
									$DolCardDav->UpdateStart( $user ); 
									
									$DolCardDav->update( $user ); 
								}
								elseif ($action == 'COMPANY_CREATE'){
								
									$DolCardDav->statut = (int)1;
									$DolCardDav->fk_user = (int)$user->id;
									$DolCardDav->fk_user_carddav = (int)$objp->rowid;

									$DolCardDav->create( $user ); 
								}

								$DolCardDav->UpdateEnd( $user ); 

						}
						elseif ($action == 'COMPANY_DELETE' && $conf->global->CARDDAV_DELETE_SERVER == 1){
						
							$res = $DolCardDav->fetch_from($user->id, '', $object->id, 0 ,'', false, $objp->rowid); 

							if($res >  0)
								if( ! @$carddav->delete($DolCardDav->carddav_id) )
									return false;
						}

						$i++;
					}
				}
        
		return 0;
    }

}
?>
