<?php

/* Copyright (C) 2016		 Oscss-Shop       <support@oscss-shop.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 * or see http://www.gnu.org/
 */

// dol_include_once('/compta/facture/class/facture.class.php'); // used for constant
// dol_include_once('/product/class/product.class.php'); // used for invoice supplier
// 


namespace CORE;
dol_include_once('/product/class/product.class.php');


namespace CORE\FRAMEWORK;
use \CORE\FRAMEWORK\Product as Product ;


class Product 
	extends \Product{

    public $OSE_loaded_version = DOL_VERSION;


    	/**
	 * Check TYPE constants
	 * @var int
	 */
	var $type = self::TYPE_PRODUCT;
	
	
	/**
	 * Regular product
	 */
	const TYPE_PRODUCT = 0;
	/**
	 * Service
	 */
	const TYPE_SERVICE = 1;
	
	
    /**
     *  Load a product in memory from database
     *
     * @param  int    $id                Id of product/service to load
     * @param  string $ref               Ref of product/service to load
     * @param  string $ref_ext           Ref ext of product/service to load
     * @param  string $barcode           Barcode of product/service to load
     * @param  int    $ignore_expression Ignores the math expression for calculating price and uses the db value instead
     * @return int                         <0 if KO, 0 if not found, >0 if OK
     */
    function fetch($id='', $ref='', $ref_ext='', $barcode='', $ignore_expression=0)
    {
        include_once DOL_DOCUMENT_ROOT.'/core/lib/company.lib.php';

        global $langs, $conf;

        dol_syslog(get_class($this)."::fetch id=".$id." ref=".$ref." ref_ext=".$ref_ext);

        // Check parameters
        if (! $id && ! $ref && ! $ref_ext && ! $barcode) {
            $this->error='ErrorWrongParameters';
            dol_syslog(get_class($this)."::fetch ".$this->error);
            return -1;
        }

        $sql = "SELECT rowid, ref, ref_ext, label, description, url, note as note_private, customcode, fk_country, price, price_ttc,";
        $sql.= " price_min, price_min_ttc, price_base_type, cost_price, default_vat_code, tva_tx, recuperableonly as tva_npr, localtax1_tx, localtax2_tx, localtax1_type, localtax2_type, tosell,";
        $sql.= " tobuy, fk_product_type, duration, fk_default_warehouse, seuil_stock_alerte, canvas, weight, weight_units,";
        $sql.= " length, length_units, width, width_units, height, height_units,";
        $sql.= " surface, surface_units, volume, volume_units, barcode, fk_barcode_type, finished,";
        $sql.= " accountancy_code_buy, accountancy_code_sell, accountancy_code_sell_intra, accountancy_code_sell_export, stock, pmp,";
        $sql.= " datec, tms, import_key, entity, desiredstock, tobatch, fk_unit,";
        $sql.= " fk_price_expression, price_autogen";
        $sql.= " FROM ".MAIN_DB_PREFIX."product";
        if ($id) { $sql.= " WHERE rowid = ".$this->db->escape($id);
        } else
        {
            $sql.= " WHERE entity IN (".getEntity($this->element).")";
            if ($ref) { $sql.= " AND ref = '".$this->db->escape($ref)."'";
            } else if ($ref_ext) { $sql.= " AND ref_ext = '".$this->db->escape($ref_ext)."'";
            } else if ($barcode) { $sql.= " AND barcode = '".$this->db->escape($barcode)."'";
            }
        }
// echo $sql; 
        $resql = $this->db->query($sql);
        if ($resql ) {
            if ($this->db->num_rows($resql) > 0) {
                $obj = $this->db->fetch_object($resql);

                $this->id                            = $obj->rowid;
                $this->ref                            = $obj->ref;
                $this->ref_ext                        = $obj->ref_ext;
                $this->label                        = $obj->label;
                $this->description                    = $obj->description;
                $this->url                            = $obj->url;
                $this->note_private                    = $obj->note_private;
                $this->note                            = $obj->note_private;  // deprecated

                $this->type                            = $obj->fk_product_type;
                $this->status                        = $obj->tosell;
                $this->status_buy                    = $obj->tobuy;
                $this->status_batch                    = $obj->tobatch;

                $this->customcode                    = $obj->customcode;
                $this->country_id                    = $obj->fk_country;
                $this->country_code                    = getCountry($this->country_id, 2, $this->db);
                $this->price                        = $obj->price;
                $this->price_ttc                    = $obj->price_ttc;
                $this->price_min                    = $obj->price_min;
                $this->price_min_ttc                = $obj->price_min_ttc;
                $this->price_base_type                = $obj->price_base_type;
                $this->cost_price                    = $obj->cost_price;
                $this->default_vat_code                = $obj->default_vat_code;
                $this->tva_tx                        = $obj->tva_tx;
                //! French VAT NPR
                $this->tva_npr                        = $obj->tva_npr;
                $this->recuperableonly                = $obj->tva_npr;       // For backward compatibility
                //! Local taxes
                $this->localtax1_tx                    = $obj->localtax1_tx;
                $this->localtax2_tx                    = $obj->localtax2_tx;
                $this->localtax1_type                = $obj->localtax1_type;
                $this->localtax2_type                = $obj->localtax2_type;

                $this->finished                        = $obj->finished;
                $this->duration                        = $obj->duration;
                $this->duration_value                = substr($obj->duration, 0, dol_strlen($obj->duration)-1);
                $this->duration_unit                = substr($obj->duration, -1);
                $this->canvas                        = $obj->canvas;
                $this->weight                        = $obj->weight;
                $this->weight_units                    = $obj->weight_units;
                $this->length                        = $obj->length;
                $this->length_units                    = $obj->length_units;
                $this->width                        = $obj->width;
                $this->width_units                    = $obj->width_units;
                $this->height                        = $obj->height;
                $this->height_units                    = $obj->height_units;

                $this->surface                        = $obj->surface;
                $this->surface_units                = $obj->surface_units;
                $this->volume                        = $obj->volume;
                $this->volume_units                    = $obj->volume_units;
                $this->barcode                        = $obj->barcode;
                $this->barcode_type                    = $obj->fk_barcode_type;

                $this->accountancy_code_buy            = $obj->accountancy_code_buy;
                $this->accountancy_code_sell        = $obj->accountancy_code_sell;
                $this->accountancy_code_sell_intra    = $obj->accountancy_code_sell_intra;
                $this->accountancy_code_sell_export    = $obj->accountancy_code_sell_export;

                $this->fk_default_warehouse            = $obj->fk_default_warehouse;
                $this->seuil_stock_alerte            = $obj->seuil_stock_alerte;
                $this->desiredstock                    = $obj->desiredstock;
                $this->stock_reel                    = $obj->stock;
                $this->pmp                            = $obj->pmp;

                $this->date_creation                = $obj->datec;
                $this->date_modification            = $obj->tms;
                $this->import_key                    = $obj->import_key;
                $this->entity                        = $obj->entity;

                $this->ref_ext                        = $obj->ref_ext;
                $this->fk_price_expression            = $obj->fk_price_expression;
                $this->fk_unit                        = $obj->fk_unit;
                $this->price_autogen                = $obj->price_autogen;

                $this->db->free($resql);

                // Retreive all extrafield
                // fetch optionals attributes and labels
                $this->fetch_optionals();

                // multilangs
                if (! empty($conf->global->MAIN_MULTILANGS)) { $this->getMultiLangs();
                }

                // Load multiprices array
                if (! empty($conf->global->PRODUIT_MULTIPRICES))                // prices per segment
                {
                    for ($i=1; $i <= $conf->global->PRODUIT_MULTIPRICES_LIMIT; $i++)
                    {
                        $sql = "SELECT price, price_ttc, price_min, price_min_ttc,";
                        $sql.= " price_base_type, tva_tx, default_vat_code, tosell, price_by_qty, rowid, recuperableonly";
                        $sql.= " FROM ".MAIN_DB_PREFIX."product_price";
                        $sql.= " WHERE entity IN (".getEntity('productprice').")";
                        $sql.= " AND price_level=".$i;
                        $sql.= " AND fk_product = ".$this->id;
                        $sql.= " ORDER BY date_price DESC, rowid DESC";
                        $sql.= " LIMIT 1";
                        $resql = $this->db->query($sql);
                        if ($resql) {
                            $result = $this->db->fetch_array($resql);

                            $this->multiprices[$i]=$result["price"];
                            $this->multiprices_ttc[$i]=$result["price_ttc"];
                            $this->multiprices_min[$i]=$result["price_min"];
                            $this->multiprices_min_ttc[$i]=$result["price_min_ttc"];
                            $this->multiprices_base_type[$i]=$result["price_base_type"];
                            // Next two fields are used only if PRODUIT_MULTIPRICES_USE_VAT_PER_LEVEL is on
                            $this->multiprices_tva_tx[$i]=$result["tva_tx"];     // TODO Add ' ('.$result['default_vat_code'].')'
                            $this->multiprices_recuperableonly[$i]=$result["recuperableonly"];

                            // Price by quantity
                            /*
                            $this->prices_by_qty[$i]=$result["price_by_qty"];
                            $this->prices_by_qty_id[$i]=$result["rowid"];
                            // Récuperation de la liste des prix selon qty si flag positionné
                            if ($this->prices_by_qty[$i] == 1)
                            {
                            $sql = "SELECT rowid, price, unitprice, quantity, remise_percent, remise, price_base_type";
                            $sql.= " FROM ".MAIN_DB_PREFIX."product_price_by_qty";
                            $sql.= " WHERE fk_product_price = ".$this->prices_by_qty_id[$i];
                            $sql.= " ORDER BY quantity ASC";
                            $resultat=array();
                            $resql = $this->db->query($sql);
                            if ($resql)
                            {
                            $ii=0;
                            while ($result= $this->db->fetch_array($resql)) {
                            $resultat[$ii]=array();
                            $resultat[$ii]["rowid"]=$result["rowid"];
                            $resultat[$ii]["price"]= $result["price"];
                            $resultat[$ii]["unitprice"]= $result["unitprice"];
                            $resultat[$ii]["quantity"]= $result["quantity"];
                            $resultat[$ii]["remise_percent"]= $result["remise_percent"];
                            $resultat[$ii]["remise"]= $result["remise"];                    // deprecated
                            $resultat[$ii]["price_base_type"]= $result["price_base_type"];
                            $ii++;
                            }
                            $this->prices_by_qty_list[$i]=$resultat;
                            }
                            else
                            {
                            dol_print_error($this->db);
                            return -1;
                            }
                            }*/
                        }
                        else
                        {
                            dol_print_error($this->db);
                            return -1;
                        }
                    }
                }
                elseif (! empty($conf->global->PRODUIT_CUSTOMER_PRICES))            // prices per customers
                {
                    // Nothing loaded by default. List may be very long.
                }
                else if (! empty($conf->global->PRODUIT_CUSTOMER_PRICES_BY_QTY))    // prices per quantity
                {
                    $sql = "SELECT price, price_ttc, price_min, price_min_ttc,";
                    $sql.= " price_base_type, tva_tx, default_vat_code, tosell, price_by_qty, rowid";
                    $sql.= " FROM ".MAIN_DB_PREFIX."product_price";
                    $sql.= " WHERE fk_product = ".$this->id;
                    $sql.= " ORDER BY date_price DESC, rowid DESC";
                    $sql.= " LIMIT 1";
                    $resql = $this->db->query($sql);
                    if ($resql) {
                        $result = $this->db->fetch_array($resql);

                        // Price by quantity
                        $this->prices_by_qty[0]=$result["price_by_qty"];
                        $this->prices_by_qty_id[0]=$result["rowid"];
                        // Récuperation de la liste des prix selon qty si flag positionné
                        if ($this->prices_by_qty[0] == 1) {
                            $sql = "SELECT rowid,price, unitprice, quantity, remise_percent, remise, remise, price_base_type";
                            $sql.= " FROM ".MAIN_DB_PREFIX."product_price_by_qty";
                            $sql.= " WHERE fk_product_price = ".$this->prices_by_qty_id[0];
                            $sql.= " ORDER BY quantity ASC";
                            $resultat=array();
                            $resql = $this->db->query($sql);
                            if ($resql) {
                                      $ii=0;
                                while ($result= $this->db->fetch_array($resql)) {
                                    $resultat[$ii]=array();
                                    $resultat[$ii]["rowid"]=$result["rowid"];
                                    $resultat[$ii]["price"]= $result["price"];
                                    $resultat[$ii]["unitprice"]= $result["unitprice"];
                                    $resultat[$ii]["quantity"]= $result["quantity"];
                                    $resultat[$ii]["remise_percent"]= $result["remise_percent"];
                                    //$resultat[$ii]["remise"]= $result["remise"];                    // deprecated
                                    $resultat[$ii]["price_base_type"]= $result["price_base_type"];
                                    $ii++;
                                }
                                    $this->prices_by_qty_list[0]=$resultat;
                            }
                            else
                            {
                                    dol_print_error($this->db);
                                    return -1;
                            }
                        }
                    }
                    else
                    {
                        dol_print_error($this->db);
                        return -1;
                    }
                }
                else if (! empty($conf->global->PRODUIT_CUSTOMER_PRICES_BY_QTY_MULTIPRICES))    // prices per customer and quantity
                {
                    for ($i=1; $i <= $conf->global->PRODUIT_MULTIPRICES_LIMIT; $i++)
                    {
                        $sql = "SELECT price, price_ttc, price_min, price_min_ttc,";
                        $sql.= " price_base_type, tva_tx, default_vat_code, tosell, price_by_qty, rowid, recuperableonly";
                        $sql.= " FROM ".MAIN_DB_PREFIX."product_price";
                        $sql.= " WHERE entity IN (".getEntity('productprice').")";
                        $sql.= " AND price_level=".$i;
                        $sql.= " AND fk_product = ".$this->id;
                        $sql.= " ORDER BY date_price DESC, rowid DESC";
                        $sql.= " LIMIT 1";
                        $resql = $this->db->query($sql);
                        if ($resql) {
                            $result = $this->db->fetch_array($resql);

                            $this->multiprices[$i]=$result["price"];
                            $this->multiprices_ttc[$i]=$result["price_ttc"];
                            $this->multiprices_min[$i]=$result["price_min"];
                            $this->multiprices_min_ttc[$i]=$result["price_min_ttc"];
                            $this->multiprices_base_type[$i]=$result["price_base_type"];
                            // Next two fields are used only if PRODUIT_MULTIPRICES_USE_VAT_PER_LEVEL is on
                            $this->multiprices_tva_tx[$i]=$result["tva_tx"];     // TODO Add ' ('.$result['default_vat_code'].')'
                            $this->multiprices_recuperableonly[$i]=$result["recuperableonly"];

                            // Price by quantity
                            $this->prices_by_qty[$i]=$result["price_by_qty"];
                            $this->prices_by_qty_id[$i]=$result["rowid"];
                            // Récuperation de la liste des prix selon qty si flag positionné
                            if ($this->prices_by_qty[$i] == 1) {
                                         $sql = "SELECT rowid, price, unitprice, quantity, remise_percent, remise, price_base_type";
                                         $sql.= " FROM ".MAIN_DB_PREFIX."product_price_by_qty";
                                         $sql.= " WHERE fk_product_price = ".$this->prices_by_qty_id[$i];
                                         $sql.= " ORDER BY quantity ASC";
                                         $resultat=array();
                                         $resql = $this->db->query($sql);
                                if ($resql) {
                                    $ii=0;
                                    while ($result= $this->db->fetch_array($resql)) {
                                                       $resultat[$ii]=array();
                                                       $resultat[$ii]["rowid"]=$result["rowid"];
                                                       $resultat[$ii]["price"]= $result["price"];
                                                       $resultat[$ii]["unitprice"]= $result["unitprice"];
                                                       $resultat[$ii]["quantity"]= $result["quantity"];
                                                       $resultat[$ii]["remise_percent"]= $result["remise_percent"];
                                                       $resultat[$ii]["remise"]= $result["remise"];                    // deprecated
                                                       $resultat[$ii]["price_base_type"]= $result["price_base_type"];
                                                       $ii++;
                                    }
                                    $this->prices_by_qty_list[$i]=$resultat;
                                }
                                else
                                         {
                                    dol_print_error($this->db);
                                    return -1;
                                }
                            }
                        }
                        else
                        {
                            dol_print_error($this->db);
                            return -1;
                        }
                    }
                }

                if (!empty($conf->dynamicprices->enabled) && !empty($this->fk_price_expression) && empty($ignore_expression)) {
                       include_once DOL_DOCUMENT_ROOT.'/product/dynamic_price/class/price_parser.class.php';
                    $priceparser = new PriceParser($this->db);
                       $price_result = $priceparser->parseProduct($this);
                    if ($price_result >= 0) {
                        $this->price = $price_result;
                        // Calculate the VAT
                        $this->price_ttc = price2num($this->price) * (1 + ($this->tva_tx / 100));
                        $this->price_ttc = price2num($this->price_ttc, 'MU');
                    }
                }

                // We should not load stock during the fetch. If someone need stock of product, he must call load_stock after fetching product.
                // Instead we just init the stock_warehouse array
                $this->stock_warehouse = array();

                return 1;
            }
            else
            {
                return 0;
            }
        }
        else
        {
            dol_print_error($this->db);
            return -1;
        }
    }


		/**
			@fn 
			@brief 
		*/
		public function __call($name, $args) {
		
			if(preg_match('#(update|create|delete)#', $name))  {
			print_r($args); 
			
			$result = 0; 
				$key = strtoupper('prev_'.__CLASS__.'_'.$name);
     // Call trigger
			$result=$this->call_trigger('PREV_'.$key,$user);
			if ($result < 0) { $error++; }
			// End call triggers
			
			if ($result >= 0)
				$result = parent::$name($user,$notrigger);
				if ($result < 0) { $error++; }
// 			    var_dump(__file__); 
//     exit; 
    
			// Call trigger
			if ($result >= 0)
			$result=$this->call_trigger('POST_'.$key,$user);
			if ($result < 0) { $error++; }
			// End call triggers
			
			return $result;
			}
			return false;
		}
	
	
		public function create($user,$notrigger=0){
    
			$result = 0; 
    
     // Call trigger
			$result=$this->call_trigger('PREV_PRODUCT_MODIFY',$user);
			if ($result < 0) { $error++; }
			// End call triggers
			
			if ($result >= 0)
				$result = parent::create($user,$notrigger);
				if ($result < 0) { $error++; }
// 			    var_dump(__file__); 
//     exit; 
    
			// Call trigger
			if ($result >= 0)
			$result=$this->call_trigger('POST_PRODUCT_MODIFY',$user);
			if ($result < 0) { $error++; }
			// End call triggers
			
			return $result;
		}
    public function update($id, $user, $notrigger=false, $action='update'){
    
			$result = 0; 
    
     // Call trigger
			$result=$this->call_trigger('PREV_PRODUCT_MODIFY',$user);
			if ($result < 0) { $error++; }
			// End call triggers
			
			if ($result >= 0)
				$result = parent::update($id, $user, $notrigger, $action);
				if ($result < 0) { $error++; }
// 			    var_dump(__file__); 
//     exit; 
    
			// Call trigger
			if ($result >= 0)
			$result=$this->call_trigger('POST_PRODUCT_MODIFY',$user);
			if ($result < 0) { $error++; }
			// End call triggers
			
			return $result;
		}
    

}
