<?php

include_once 'trait/fixverloader.trait.php';

/**
 * Get information on an object according to version of Dolibarr
 * @usage dol_include_once('/framework/class/fixver.class.php');
 * @usage $fv = fixver::getInstance();
 * @usage $class = $fv->name('order')->class;
 * @usage $class = $fv->GetClassByName('order');
 */
class fixver {

    use fixverLoader;

    /**
     * The Singleton's instance is stored in a static field. This field is an
     * array, because we'll allow our Singleton to have subclasses. Each item in
     * this array will be an instance of a specific Singleton's subclass. You'll
     * see how this works in a moment.
     */
    private static $instance;
    public
            $tbl   = array(),
            $col   = array(),
            $corss = array(),
            $dolv, $shortdolv, $maindolv;

    /**
     * 	@brief load know objects
     */
    public static function getInstance() {
        global $conf;
        if (!isset(self::$instance)) {
            self::$instance = new static();
        }

        // singleton
        $fv = &self::$instance;

        // 1.2.3
        $fv->dolv      = DOL_VERSION;
        // 1.2
        $fv->shortdolv = substr(DOL_VERSION, 0, strpos(DOL_VERSION, '.', strpos(DOL_VERSION, '.') + 1));
        // 1
        $fv->maindolv  = substr(DOL_VERSION, 0, strpos(DOL_VERSION, '.'));

        // load default values;
        $fv->loadDefaultCorss();

        // load default values;
        $fv->loadVersionCorss();

        // loop for hooks
        foreach ($conf->modules_parts['hooks'] as $k => $r) {
            if (!isset($fv->corss[$k])) {
                if (dol_buildpath('/' . $k . '/class/' . $k . '.class.php', 1) != false) {
                    $fv->corss[] = new fvChild($k);
                }
            }
        }

        // loop For greffon
        if (isset($conf->modules_parts['greffon']) && is_array($conf->modules_parts['greffon'])) {
            foreach ($conf->modules_parts['greffon'] as $k => $r) {
                if (isset($conf->modules_parts['hooks'][$r[0]])) {
                    if (dol_buildpath('/' . $r[0] . '/class/' . $k . '.class.php', 1) != false) {
                        $tmpChild       = new fvChild($k);
                        $tmpChild->setPathAndRelated(strtolower($r[0]));
                        $tmpChild->lang = $k . '@' . $r[0];
                        $fv->corss[]    = $tmpChild;
                    }
                }
            }
        }
        return self::$instance;
    }

    /**
     * 	@fn getTable() : string
     * 	@brief
     * 	@param string $table
     * 	@return string Table name
     */
    public function getTable($table) {
        if (isset(self::$instance->tbl[$table])) {
            return self::$instance->tbl[$table];
        }
        return $table;
    }

    /**
     * Get the changing name of columns according to version
     * 	@fn getColumn($table, $column) : string
     *  @brief THE ORDER OF THE PARAMETERS HAVE BEEN CHANGED !!!
     * 	@param string $table
     * 	@param string $column
     *  @return string Column name
     */
    public function getColumn($table, $column) {
//        $table = self::$instance->getTbl($table);
        if (isset(self::$instance->col[$column])) {
            return self::$instance->col[$column];
        } elseif (isset(self::$instance->col[$table]) && isset(self::$instance->col[$table][$column])) {
            return self::$instance->col[$table][$column];
        }
        return $column;
    }

    /**
     * DOCUMENT
     * @fn __call($method, $arguments) : fvChild
     * @return fvChild
     */
    function __call($method, $arguments) {
        $fct     = strtolower($method);
        // given parameter
        $arg     = (string) $arguments[0];
        $matches = null;
        if (preg_match('#Get([A-Z][a-z]*)By([A-Z][a-z]*)#', $method, $matches)) {
            // old method call
            $fct   = strtolower($matches[2]);
            $value = strtolower($matches[1]);
            $fv    = self::$instance;
            return $fv->$fct($arg)->$value;
        } else {
            // new method
            foreach (self::$instance->corss as &$fvChild) {
                if (isset($fvChild->$fct) && $fvChild->$fct === $arg) {
                    return $fvChild;
                }
            }
            // no details for this $arg, we send a default
            $child = new fvChild($arg);
            dol_syslog("fixver $fct($arg) not found, returned default object");
            return $child;
        }
    }

    /**
     * Singletons should not be cloneable.
     */
    protected function __clone() {
        throw new Exception("Cannot clone a singleton.");
    }

    /**
     * Singletons should not be restorable from strings.
     */
    public function __wakeup() {
        throw new Exception("Cannot unserialize a singleton.");
    }

    /**
     * Singletons should not be called by new Class().
     */
    protected function __construct() {

    }

}

class fvChild {

    // public vars
    var $type;                      // inserted in element to element
    var $name;                      // the name of the object
    var $class;                     // the Class name
    var $title;                     // the title
    var $table;                     // the table
    var $tableline;                 // the line table
    var $datefieldname = 'datec';   // the field name
    var $path;                      // the relative path of files
    var $disableamount = 1;         // disable amounts
    var $nameid        = 'id';      // the $_GET param name
    var $tabs;                      // the tabs name
    var $rights;                    // the right name in $user->rights->$rights
    var $lang;                      // the related
    var $list;                      // the list of objects URI
    var $card;                      // the card of object URI
    var $margin        = 'minus';   // no idea
    var $oldtype;                   // oldtype but dangerous, old since when ???
    // main var
    var $element;

    public function __construct(string $object) {
        global $db;
        $this->element = strtolower($object);

        // default values
        $this->oldtype   = $object;
        $this->class     = ucfirst($this->element);
        $this->name      = ucfirst($this->element);
        $this->lang      = $this->element;
        $this->rights    = $this->element;
        $this->table     = $this->element;
        $this->tableline = $this->element . 'Line';
        $this->tabs      = $this->element;
        $this->title     = ucfirst($this->element);
        $this->type      = $this->element;
        if (class_exists($this->class)) {
            // get info from class
            $className = $this->class;
            try {
                $staticObj = new $className($db);
                if (isset($staticObj->table_element)) {
                    $this->table = $staticObj->table_element;
                }
                if (isset($staticObj->table_element_line)) {
                    $this->tableline = $staticObj->table_element_line;
                }
            } catch (Exception $exc) {

            }
        }
        // directories
        $this->setPathAndRelated($this->element . "/");
    }

    // always return something, give the object type if asked an unknown field
    public function __get($name) {
        return $this->element;
    }

    /**
     * DOCUMENT
     * @fn setPath(string $path) : void
     * @param string $path ...
     */
    function setPathAndRelated(string $path) {
        $this->path = $path;
        $this->list = $this->path . "list.php";
        $this->card = $this->path . "card.php";
    }

}
