<?php

require_once DOL_DOCUMENT_ROOT . '/core/class/commonobject.class.php';
require_once DOL_DOCUMENT_ROOT . '/user/class/user.class.php';
require_once DOL_DOCUMENT_ROOT . '/user/class/usergroup.class.php';

/** Groups Handle list of users from groups
 * - sorting
 * - remove doublons
 * @param object $db global
 */
class Groups extends CommonObject {

    var $groups = array();
    var $users  = array();

    /** @param object $db global
     */
    function __construct($db) {
        $this->db = $db;
    }

    /** fetchGroups Load groups by array of Ids
     * @param array $groupsId Ids of groups
     * @param $crashOnUnknown return -2 if group unknown
     * @return boolean
     */
    function fetchGroups(array $groupsId = array(), $crashOnUnknown = false) {
        //init
        $this->initLists();
        // check
        if (!is_array($groupsId) || count($groupsId) == 0) {
            return -1;
        }

        foreach ($groupsId as $groupId) {
            $obj = new UserGroup($this->db);
            $obj->fetch($groupId);
            if ($obj->id) {
                $this->groups[] = $obj;
                $this->fetchUsers($obj->members);
            } elseif ($crashOnUnknown) {
                $this->initLists();
                return -2;
            }
        }
        // clean users
        $this->clean();
        return true;
    }

    /** clean create a notification and send it to the ...
     * @param string $variable theDoc
     * @return FALSE or int for the id of the Notification
     */
    function clean($variable = 'users') {
        $this->removeDoublons($variable);
        $this->orderObject($variable);
        return true;
    }

    /** fetchUser add a user to the list
     * @param User/int $user a user to add, or by id, or by User object
     * @return True
     */
    function fetchUser($user) {
        if (is_object($user))
            $this->users[] = $user;
        else { // fetch a user
            $obj = new User($this->db);
            $obj->fetch($user);
            if ($obj->id) {
                $this->users[] = $obj;
            } return -1;
        }
        return true;
    }

    /** fetchGroup add a group to the list
     * @param User/int $group a user to add, or by id, or by UserGroup object
     * @return True
     */
    function fetchGroup($group) {
        if (is_object($group))
            $this->groups[] = $group;
        else { // fetch a user
            $obj = new UserGroup($this->db);
            $obj->fetch($group);
            if ($obj->id) {
                $this->groups[] = $obj;
            }
        }
        return true;
    }

    /** fetchUsers add a user to the list or by id or by object User
     * @param array $users
     * @return True
     */
    function fetchUsers(array $users=array()) {
        foreach ($users as $user) {
            $this->fetchUser($user);
        }
        return true;
    }

    /** removeDoublon remove doublons
     * @param string $var work on users or groups
     * @return true
     */
    function removeDoublons($variable = 'users') {
        // remove doublons
        $objId    = array();
        $tmpArray = array();
        foreach ($this->$variable as $obj) {
            if (!in_array($obj->id, $objId)) {
                $objId[]    = $obj->id;
                $tmpArray[] = $obj;
            }
        }
        $this->$variable = $tmpArray;
        return true;
    }

    /** orderObject sort $users or $gorup
     * @param string $variable theDoc
     * @return FALSE or int for the id of the Notification
     */
    function orderObject($variable = 'users') {
        if ($variable == 'users')
            usort($this->$variable, array($this, '_compareLastname'));
        else
            usort($this->$variable, array($this, '_compareName'));
        return true;
    }

    /** compareLastname compare objects on lastname + firstname
     * @return boolean
     */
    private function _compareLastname($a, $b) {
        $res = strcmp($a->lastname, $b->lastname);
        if ($res != 0)
            return $res;
        else
            return strcmp($a->firstname, $b->firstname);
    }

    /** compareName compare objects on name
     * @return boolean
     */
    private function _compareName($a, $b) {
        return strcmp($a->name, $b->name);
    }

    /** initLists empty the list of users and groups
     * @return void
     */
    function initLists() {
        $this->groups = array();
        $this->users  = array();
    }

}
