<?php
/* Copyright (C) 2007-2017 Laurent Destailleur  <eldy@users.sourceforge.net>
 * Copyright (C) ---Put here your own copyright and developer email---
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * \file    test/unit/apiregistrationTest.php
 * \ingroup logistics
 * \brief   PHPUnit test for apiregistration class.
 */

namespace test\unit;

global $conf,$user,$langs,$db;
require_once dirname(__FILE__).'/../../../../master.inc.php';
require_once dirname(__FILE__).'/../../class/apiregistration.class.php';
require_once dirname(__FILE__).'/../../../../core/lib/admin.lib.php';
if (empty($user->id)) {
    print "Load permissions for admin user nb 1\n";
    $user->fetch(1);
    $user->getrights();
}
$conf->global->MAIN_DISABLE_ALL_MAILS=1;


/**
 * Class apiregistrationTest
 * @package Testlogistics
 */
class apiregistrationTest extends \PHPUnit_Framework_TestCase
{
    protected $savconf;
    protected $savuser;
    protected $savlangs;
    protected $savdb;
    
    
    
     protected $cstname='';

    /**
     * Constructor
     * We save global variables into local variables
     *
     * @return apiregistrationTest
     */
    public function __construct($name = NULL, array $data = array(), $dataName = '') {
		parent::__construct($name, $data, $dataName);

    	//$this->sharedFixture
        global $conf,$user,$langs,$db;
        $this->savconf=$conf;
        $this->savuser=$user;
        $this->savlangs=$langs;
        $this->savdb=$db;

        print __METHOD__." db->type=".$db->type." user->id=".$user->id;
        //print " - db ".$db->db;
        print "\n";
    }

    // Static methods
    public static function setUpBeforeClass()
    {
        global $conf,$user,$langs,$db;
//         $db->begin(); // This is to have all actions inside a transaction even if test launched without suite.


        if (empty($conf->framework->enabled)) { print __METHOD__." module framework order must be enabled.\n"; 

        die(); }
        

        if(!file_exists(  dol_buildPath( '/framework/core/build.json', 0) ) ) { print __METHOD__." core/build.json no found.\n"; 

        die(); }

        print __METHOD__."\n";
    }

    // tear down after class
    public static function tearDownAfterClass()
    {
        global $conf,$user,$langs,$db;
//         $db->rollback();

        print __METHOD__."\n";
    }
    /**
     * testapiregistrationCreate
     *
     * @return  void
     */
    public function testapiregistrationNewObject()
    {
        global $conf,$user,$langs,$db;
        $conf=$this->savconf;
        $user=$this->savuser;
        $langs=$this->savlangs;
        $db=$this->savdb;

		$build = json_decode( file_get_contents(dol_buildPath( '/framework/core/build.json', 0) ) );

		if(is_object($build)){
// 			global $conf;
			
			// editor
			$this->editor_name = $build->editor->name;
			// For update linked
			$this->urlapi = $build->editor->api; 
			$this->api_key = $build->editor->apikey; 

			
			$this->cstname = "FRAMEWORKAPIKEYLINK".strtoupper(str_replace(' ', '', trim($this->editor_name)));
			
			$apiregis = new \ApiRegistration( $build->editor->api, $build->editor->apikey, $conf->global->{$this->cstname} );  
		
		
// 			var_dump($apiregis); 
		}
// 		exit; 
		$this->assertTrue(true);

        print __METHOD__." result=".$result."\n";
        return $apiregis;
    }

    /**
     * testapiregistrationCreate
     *
     * @return  void
     * @depends	testapiregistrationNewObject
     * The depends says test is run only if previous is ok
     */
    public function testapiregistrationCheckTestInit($apiregis)
    {
        global $conf,$user,$langs,$db;
        $conf=$this->savconf;
        $user=$this->savuser;
        $langs=$this->savlangs;
        $db=$this->savdb;

// var_dump($apiregis->CheckTestInit($conf) ); 

		$res = $apiregis->CheckTestInit($conf);
		
		if(!$res) 
			$this->assertFalse( false );
			
// 		if($res)
// 			$apiregis->data
// 		$this->assertTrue( ! );
		
		

        print __METHOD__." result=".$result."\n";
        return $apiregis;
    }

    
    /**
     * testapiregistrationCreate
     *
     * @return  void
     * @depends	testapiregistrationCheckTestInit
     * The depends says test is run only if previous is ok
     */
    public function testapiregistrationRegistrationMaster($apiregis)
    {
        global $conf,$user,$langs,$db;
        $conf=$this->savconf;
        $user=$this->savuser;
        $langs=$this->savlangs;
        $db=$this->savdb;
// 		$apiregis->verbose = true; 
		// Data transmise à l'api de l'editeur 
		$fields = array(
			'dolibarrurl' => urlencode('phpunitTest'),
			'dolibarrip' => urlencode('127.0.0.1'),
			'company' => 'oscss-shop',
			'email' => 'phpunit@oscss-shop.fr',
			'lang' => 'fr_FR'
		);

		$build = json_decode( file_get_contents(dol_buildPath( '/framework/core/build.json', 0) ) );

		if(is_object($build)){
// 			global $conf;
			
			// editor
			$this->editor_name = $build->editor->name;
			// For update linked
// 			$this->urlapi = $build->editor->api; 
			$apiregis->api_key_base = $build->editor->apikey; 

			
			$this->cstname = "FRAMEWORKAPIKEYLINK".strtoupper(str_replace(' ', '', trim($this->editor_name)));
			
			
			// initial context prev one registerer
			if($apiregis->apikey == $apiregis->api_key_base) {

				
// print_r($fields); 
				$result = $apiregis->SetRegistrationMaster( $fields ); 
// print_r($apiregis); 
				if($result == false) 
					$this->assertFalse( true , print_r($result, true) );
				
	// 			$this->assertTrue( ($result == false) );
				
				// clean constant 
				dolibarr_del_const($db,$this->cstname, $conf->entity);

				$res = dolibarr_set_const($this->savdb,$this->cstname, strtoupper($result), 'chaine', 0, '', $conf->entity);

					
					
				$this->assertTrue( ($res == 1) );
			
			}
			else 
				$result = $apiregis->apikey; 

        print __METHOD__." result=".$result."\n";
        }
        else {
//         // Stop here and mark this test as incomplete.
			$this->markTestIncomplete(
			'This test has not been implemented yet.'
			);
        }
        
        // ok TEst 
		$this->assertTrue( true );
		
// 		$this->assertTrue( false);
		$apiregis->verbose = false; 
//         print_r($apiregis); 
        return $apiregis;
    }
    
/**
     * testapiregistrationCreate
     *
     * @return  void
     * @depends	testapiregistrationRegistrationMaster
     * The depends says test is run only if previous is ok
     */
//     public function testapiregistrationCheckTestInitnewKey($apiregis)
//     {
//         global $conf,$user,$langs,$db;
//         $conf=$this->savconf;
//         $user=$this->savuser;
//         $langs=$this->savlangs;
//         $db=$this->savdb;
// 
// // var_dump($apiregis->CheckTestInit($conf) ); 
// // 		$build = json_decode( file_get_contents(dol_buildPath( '/framework/core/build.json', 0) ) );
// // 
// // 		if(is_object($build)){
// // // 			global $conf;
// // 			
// // 			// editor
// // 			$this->editor_name = $build->editor->name;
// // 			// For update linked
// // 			$this->urlapi = $build->editor->api; 
// // 			$this->api_key = $build->editor->apikey; 
// // 
// // 			
// // 			$this->cstname = "FRAMEWORKAPIKEYLINK".strtoupper(str_replace(' ', '', trim($this->editor_name)));
// // 			
// // 			$apiregis = new \ApiRegistration( $build->editor->api, $build->editor->apikey, $conf->global->{$this->cstname} );  
// // 		
// // 		
// // // 			var_dump($apiregis); 
// // 
// // 			$this->assertTrue( $apiregis->CheckTestInit($conf) );
// // 		}
// 		
// 		
// 		
// 
//         print __METHOD__." result=".$result."\n";
//         return $localobject;
//     }
    
    
    /**
     * testapiregistrationCreate
     *
     * @return  void
     * @depends	testapiregistrationRegistrationMaster
     * The depends says test is run only if previous is ok
     */
    public function testapiregistrationSetRegistrationModule($apiregis)
    {
        global $conf,$user,$langs,$db;
        $conf=$this->savconf;
        $user=$this->savuser;
        $langs=$this->savlangs;
        $db=$this->savdb;


		if(is_array($apiregis->subarray) ) {
			foreach($apiregis->subarray as $k=>$module){
			
				$fields = array(
					'module' => urlencode($module['module'])
				);
// 							print_r($fields); 
				$result = $apiregis->SetRegistrationModule($fields);
				
				// check false result 
				$this->assertFalse( ($result == false) , 'method  SetRegistrationModule retrun false');
				//check  curl error
				$this->assertFalse( (!empty($apiregis->curl_error) ), $apiregis->curl_error);
				
				//check  reponse API error
				if(isset($apiregis->curl_result['error']) )
					$this->assertFalse( (!empty($apiregis->curl_result['error']) ), $apiregis->curl_result['message']);
				else
					$this->assertFalse(false);
			}
		}

		// ok TEst 
		$this->assertTrue( true );

        print __METHOD__." result=".$result."\n";
        return $apiregis;
    }
    
    /**
     * testapiregistrationCreate
     *
     * @return  void
     * @depends	testapiregistrationRegistrationMaster
     * The depends says test is run only if previous is ok
     */
    public function testapiregistrationGetMyAccount($apiregis)
    {
        global $conf,$user,$langs,$db;
        $conf=$this->savconf;
        $user=$this->savuser;
        $langs=$this->savlangs;
        $db=$this->savdb;


			$result = $apiregis->GetMyAccount(array());
			if($result == false) 
					$this->assertFalse( true , print_r($result, true) );
			elseif(isset($result['error']) )
				$this->assertFalse( true, $result['error'] );

		// ok TEst 
		$this->assertTrue( true );

        print __METHOD__." result=".$result."\n";
        return $apiregis;
    }
	/**
	 * Unit test setup
     * @return void
	 */
	protected function setUp()
	{
        global $conf,$user,$langs,$db;
        $conf=$this->savconf;
        $user=$this->savuser;
        $langs=$this->savlangs;
        $db=$this->savdb;

// 		$this->db->begin();
        print __METHOD__."\n";
	}

	/**
	 * Verify pre conditions
     * @return void
	 */
	protected function assertPreConditions()
	{
// 		fwrite(STDOUT, __METHOD__ . "\n");
	}



	/**
	 * Verify post conditions
     * @return void
	 */
	protected function assertPostConditions()
	{
		fwrite(STDOUT, __METHOD__ . "\n");
	}

	/**
	 * Unit test teardown
     * @return void
	 */
	protected function tearDown()
	{
        global $conf,$user,$langs,$db;
        $conf=$this->savconf;
        $user=$this->savuser;
        $langs=$this->savlangs;
        $db=$this->savdb;

// 		$this->db->rollback();
// 		$db->rollback();

		

// 		fwrite(STDOUT, __METHOD__ . "\n");
		 print __METHOD__."\n";
	}


	/**
	 * Unsuccessful test
	 *
	 * @param  Exception $e    Exception
     * @return void
	 * @throws Exception
	 */
// 	protected function onNotSuccessfulTest(Exception $e)
// 	{
// 		fwrite(STDOUT, __METHOD__ . "\n");
// 		throw $e;
// 	}
}
